<%@ Application Language="C#" %>
<%@ Import Namespace="System.Collections.Generic" %>
<%@ Import Namespace="System.IO" %>
<%@ Import Namespace="BugTrapServer" %>

<script runat="server">

    /// <summary>
    /// Report file name pattern.
    /// </summary>
    [DoNotObfuscate]
    public const string reportNamePattern = "error_report_";

    /// <summary>
    /// Load directories contents from the disk.
    /// </summary>
    /// <param name="applicationSettings">Application settings.</param>
    /// <return>Unique numbers of recent bug reports.</return>
    private Dictionary<string, AppDirInfo> LoadDirectories(ApplicationSettings applicationSettings)
    {
        Dictionary<string, AppDirInfo> lastReportNumbers = new Dictionary<string, AppDirInfo>();
        Directory.CreateDirectory(applicationSettings.ReportPath);
        string[] directories = Directory.GetDirectories(applicationSettings.ReportPath);
        string reportNameFilter = global_asax.reportNamePattern + '*';
        foreach (string directory in directories)
        {
            string dirName = Path.GetFileName(directory);
            AppDirInfo appDirInfo = new AppDirInfo();
            string[] files = Directory.GetFiles(directory, reportNameFilter);
            foreach (string file in files)
            {
                string fileName = Path.GetFileNameWithoutExtension(file);
                if (!fileName.StartsWith(global_asax.reportNamePattern))
                    continue;
                int reportNumber = int.Parse(fileName.Substring(global_asax.reportNamePattern.Length));
                if (appDirInfo.MaxReportNumber < reportNumber)
                    appDirInfo.MaxReportNumber = reportNumber;
                ++appDirInfo.NumReports;
            }
            lastReportNumbers[dirName] = appDirInfo;
        }
        return lastReportNumbers;
    }

    /// <summary>
    /// Code that runs on application startup.
    /// </summary>
    /// <param name="sender">Event souurce.</param>
    /// <param name="e">Event data.</param>
    [DoNotObfuscate]
    void Application_Start(object sender, EventArgs e)
    {
        ApplicationSettings applicationSettings = (ApplicationSettings)ConfigurationManager.GetSection("applicationSettings");
        if (!Path.IsPathRooted(applicationSettings.ReportPath))
            applicationSettings.ReportPath = this.Server.MapPath(applicationSettings.ReportPath);
        this.Application["applicationSettings"] = applicationSettings;
        this.Application["lastReportNumbers"] = this.LoadDirectories(applicationSettings);
    }

    /// <summary>
    /// Code that runs on application shutdown.
    /// </summary>
    /// <param name="sender">Event souurce.</param>
    /// <param name="e">Event data.</param>
    [DoNotObfuscate]
    void Application_End(object sender, EventArgs e)
    {
    }

    /// <summary>
    /// Code that runs when an unhandled error occurs.
    /// </summary>
    /// <param name="sender">Event souurce.</param>
    /// <param name="e">Event data.</param>
    [DoNotObfuscate]
    void Application_Error(object sender, EventArgs e)
    {
    }

    /// <summary>
    /// Code that runs when a new session is started.
    /// </summary>
    /// <param name="sender">Event souurce.</param>
    /// <param name="e">Event data.</param>
    [DoNotObfuscate]
    void Session_Start(object sender, EventArgs e)
    {
    }

    /// <summary>
    /// Code that runs when a session ends.
    /// Note: The Session_End event is raised only when the sessionstate mode
    /// is set to InProc in the Web.config file. If session mode is set to StateServer
    /// or SQLServer, the event is not raised.
    /// </summary>
    /// <param name="sender">Event souurce.</param>
    /// <param name="e">Event data.</param>
    [DoNotObfuscate]
    void Session_End(object sender, EventArgs e)
    {
    }

</script>
